@@ -1,4 +1,4 @@
-/* $OpenBSD: dns.c,v 1.28 2012/05/23 03:28:28 djm Exp $ */
+/* $OpenBSD: dns.c,v 1.29 2013/05/17 00:13:13 djm Exp $ */
 
 /*
  * Copyright (c) 2003 Wesley Griffin. All rights reserved.
@@ -258,7 +258,7 @@ verify_host_key_dns(const char *hostname, struct sockaddr *address,
 
 		if (hostkey_digest_type != dnskey_digest_type) {
 			hostkey_digest_type = dnskey_digest_type;
-			xfree(hostkey_digest);
+			free(hostkey_digest);
 
 			/* Initialize host key parameters */
 			if (!dns_read_key(&hostkey_algorithm,
@@ -278,10 +278,10 @@ verify_host_key_dns(const char *hostname, struct sockaddr *address,
 			    hostkey_digest_len) == 0)
 				*flags |= DNS_VERIFY_MATCH;
 		}
-		xfree(dnskey_digest);
+		free(dnskey_digest);
 	}
 
-	xfree(hostkey_digest); /* from key_fingerprint_raw() */
+	free(hostkey_digest); /* from key_fingerprint_raw() */
 	freerrset(fingerprints);
 
 	if (*flags & DNS_VERIFY_FOUND)
@@ -324,7 +324,7 @@ export_dns_rr(const char *hostname, Key *key, FILE *f, int generic)
 			for (i = 0; i < rdata_digest_len; i++)
 				fprintf(f, "%02x", rdata_digest[i]);
 			fprintf(f, "\n");
-			xfree(rdata_digest); /* from key_fingerprint_raw() */
+			free(rdata_digest); /* from key_fingerprint_raw() */
 			success = 1;
 		}
 	}
