@@ -97,7 +97,7 @@ store_lastlog_message(const char *user, uid_t uid)
 	time_string = sys_auth_get_lastlogin_msg(user, uid);
 	if (time_string != NULL) {
 		buffer_append(&loginmsg, time_string, strlen(time_string));
-		xfree(time_string);
+		free(time_string);
 	}
 # else
 	last_login_time = get_last_login_time(uid, user, hostname,
